#!/usr/bin/python
# -*-coding:utf-8-*-

"""
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions: 
* 
* The above notice and this permission notice shall be included in all copies
* or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* File for "Alpha Blending" lesson of the OpenGL tutorial on
* www.videotutorialsrock.com
*
* File in python created by Sehnoax based on files from 
* www.videotutorialsrock.com
"""

from OpenGL.GLUT import *
from OpenGL.GLU import *
from OpenGL.GL import *
import sys
from PIL import Image
from cube import *


BOX_SIZE = 7
ALPHA = 0.6


class myGLCanvas(object):
    def __init__(self):      
        self.vs = ['']*4
        self.cube = Cube()
        self.stop = 0
        
        glutInit(sys.argv)
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        glutInitWindowSize(400, 400)
        glutCreateWindow(b"Alpha Blending - videotutorialsrock.com")

        self.initRendering()
        self.loadTexture()
        self.initCube()

        glutDisplayFunc(self.OnPaint)
        glutReshapeFunc(self.OnResize)
        glutKeyboardFunc(self.handleKeypress)

        glutTimerFunc(25, self.update, 0)        

        glutMainLoop()
        return 0
        

    def initCube(self):
        self.cube.top.up = Vecteur(0, 0, -1)
        self.cube.top.rightFace = Vecteur(1, 0, 0)
        self.cube.top.out = Vecteur(0, 1, 0)

        self.cube.bottom.up = Vecteur(0, 0, 1)
        self.cube.bottom.rightFace = Vecteur(1, 0, 0)
        self.cube.bottom.out = Vecteur(0, -1, 0)

        self.cube.left.up = Vecteur(0, 0, -1)
        self.cube.left.rightFace = Vecteur(0, 1, 0)
        self.cube.left.out = Vecteur(-1, 0, 0)

        self.cube.right.up = Vecteur(0, -1, 0)
        self.cube.right.rightFace = Vecteur(0, 0, 1)
        self.cube.right.out = Vecteur(1, 0, 0)

        self.cube.front.up = Vecteur(0, 1, 0)
        self.cube.front.rightFace = Vecteur(1, 0, 0)
        self.cube.front.out = Vecteur(0, 0, 1)

        self.cube.back.up = Vecteur(1, 0, 0)
        self.cube.back.rightFace = Vecteur(0, 1, 0)
        self.cube.back.out = Vecteur(0, 0, -1)


    def faceVertices(self, face, vs):
        self.vs[0] = BOX_SIZE/2*(face.out - face.rightFace - face.up)
        self.vs[1] = BOX_SIZE/2*(face.out - face.rightFace + face.up)
        self.vs[2] = BOX_SIZE/2*(face.out + face.rightFace + face.up)
        self.vs[3] = BOX_SIZE/2*(face.out + face.rightFace - face.up)

    
    def drawTopFace(self):
        face = self.cube.top
        self.faceVertices(face, self.vs)
        
        glDisable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)

        glColor4f(1., 1., 0., ALPHA)
        glNormal3f(face.out._x, face.out._y, face.out._z)
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()


    def drawBottomFace(self):
        face = self.cube.bottom
        self.faceVertices(face, self.vs)
        
        glDisable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)

        glColor4f(1., 0., 1., ALPHA)
        glNormal3f(face.out._x, face.out._y, face.out._z)
        
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()


    def drawLeftFace(self):
        face = self.cube.left
        self.faceVertices(face, self.vs)
        
        glDisable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)

        glNormal3f(face.out._x, face.out._y, face.out._z)
        glColor4f(0., 1., 1., ALPHA)                
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glColor4f(0., 0., 1., ALPHA)                
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()

    
    def drawRightFace(self):
        face = self.cube.right
        self.faceVertices(face, self.vs)
        
        glDisable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)

        glNormal3f(face.out._x, face.out._y, face.out._z)
        glColor4f(1., 0., 0., ALPHA)        
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glColor4f(0., 1., 0., ALPHA)        
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()


    def drawFrontFace(self):
        face = self.cube.front
        self.faceVertices(face, self.vs)

        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glColor4f(1.0, 1.0, 1.0, ALPHA)

        glBegin(GL_QUADS)

        glNormal3f(face.out._x, face.out._y, face.out._z)
        glTexCoord2f(0, 0)
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glTexCoord2f(0, 1)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glTexCoord2f(1, 1)
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glTexCoord2f(1, 0)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()


    def drawBackFace(self):
        face = self.cube.back
        self.faceVertices(face, self.vs)

        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glColor4f(1.0, 1.0, 1.0, ALPHA)

        glBegin(GL_QUADS)

        glNormal3f(face.out._x, face.out._y, face.out._z)
        glTexCoord2f(0, 0)
        glVertex3f(self.vs[0]._x, self.vs[0]._y, self.vs[0]._z)
        glTexCoord2f(0, 1)
        glVertex3f(self.vs[1]._x, self.vs[1]._y, self.vs[1]._z)
        glTexCoord2f(1, 1)
        glVertex3f(self.vs[2]._x, self.vs[2]._y, self.vs[2]._z)
        glTexCoord2f(1, 0)
        glVertex3f(self.vs[3]._x, self.vs[3]._y, self.vs[3]._z)
        
        glEnd()


    def drawFace(self, face):
        if face == self.cube.top :
            self.drawTopFace()
        elif face == self.cube.bottom :
            self.drawBottomFace()
        elif face == self.cube.left :
            self.drawLeftFace()
        elif face == self.cube.right :
            self.drawRightFace()
        elif face == self.cube.front :
            self.drawFrontFace()
        else :
            self.drawBackFace()
    

    def initRendering(self):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_COLOR_MATERIAL)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glEnable(GL_NORMALIZE)

        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        
        
    def OnPaint(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glTranslatef(0.0, 0.0, -20.0)

        ambientLight = [0.3, 0.3, 0.3, 1.0]
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight)
        
        directedLight = [0.7, 0.7, 0.7, 1.0]
        directedLightPos = [-2*BOX_SIZE, 4*BOX_SIZE, 1.0]
        glLightfv(GL_LIGHT0, GL_DIFFUSE, directedLight)
        glLightfv(GL_LIGHT0, GL_POSITION, directedLightPos)

        faces = []
        faces.append(self.cube.top)
        faces.append(self.cube.bottom)
        faces.append(self.cube.left)
        faces.append(self.cube.right)
        faces.append(self.cube.front)
        faces.append(self.cube.back)

        for face in faces :
            self.drawFace(face)       
	
        glutSwapBuffers()
          

    def OnResize(self, w, h):
        glViewport(0, 0, w, h)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        gluPerspective(45., \
                       float(w)/float(h), \
                       1., \
                       200.)
        
    
    def update(self, value):
        self.cube.rotateCube(Vecteur(1., 1., 0), 1.)
        
        glutPostRedisplay()
        glutTimerFunc(25, self.update, 0)
        

    def handleKeypress(self, key, x, y):
        if ord(key) == 27 : exit()


    def loadTexture(self):
        self.textureId = glGenTextures(1)
        
        try:
            image = Image.open("vtr.bmp")
        except:
            print ("Chargement de la texture impossible : fichier introuvable")
            return

        w, h = image.size
        pixels = image.transpose(Image.FLIP_TOP_BOTTOM).tobytes()
        
        glBindTexture(GL_TEXTURE_2D, self.textureId)
        glTexImage2D(GL_TEXTURE_2D, \
                     0, \
                     GL_RGB, \
                     w, h, \
                     0, \
                     GL_RGB, \
                     GL_UNSIGNED_BYTE, \
                     pixels)

        del image
        del pixels
        
 
if __name__ == '__main__':
    canvas = myGLCanvas()
